"""
    Created on 2013-03-03
    @author: jldupont
"""
__all__=["push_event"]

import unittest

import os, sys
ap=os.path.abspath(__file__)
dn=os.path.dirname
base=dn(dn(dn(ap)))
sys.path.insert(0, base)
from subpub import sub, pub, upub, get_subs



def push_event(e):
    global EVENTS
    EVENTS.append(e)

@sub
def on_test1(param):   
    global EVENTS
    
    EVENTS.append(("test1", param))

@sub
def on_test2(param1, param2):
    global EVENTS
    
    EVENTS.append(("test2", (param1, param2)))

@sub
def on_test3(param1):
    global EVENTS
    
    EVENTS.append(("test3", param1))
    
    ## the next one goes on the queue, at the end, as normal
    pub("test5",  param1)
    
    ## but this one would be queued in front of the previous
    upub("test4", param1)
    


@sub
def on_test4(param1):
    global EVENTS
    
    EVENTS.append(("test4", param1))

@sub
def on_test5(param1):
    global EVENTS
    
    EVENTS.append(("test5", param1))

    
@sub
def on_test6():
    raise Exception("Some exception in test6")


@sub
def on_test99(param):   
    global EVENTS
    
    EVENTS.append(("test99", param))


class TestCases(unittest.TestCase):
    
    def setUp(self):
        global EVENTS
        EVENTS=[]
    
            
    
    def test_a1_simple(self):
        """
        Simple 1 topic, 2 msgs
        """
        global EVENTS
        
        pub("test1", "value1")
        pub("test1", "value2")

        self.assertEqual(len(EVENTS), 2)
        
        topic1, val1=EVENTS.pop(0)
        self.assertEqual(topic1, "test1")
        self.assertEqual(val1, "value1")
        
        topic2, val2=EVENTS.pop(0)
        self.assertEqual(topic2, "test1")
        self.assertEqual(val2, "value2")
        
    def test_a2_two_topics(self):
        """
        Two topics, 2 msgs
        """
        
        pub("test1", "value1")
        pub("test2", "param1", "param2")
        self.assertEqual(len(EVENTS), 2)

        topic1, val1=EVENTS.pop(0)
        self.assertEqual(topic1, "test1")
        self.assertEqual(val1, "value1")
        
        topic2, val2=EVENTS.pop(0)
        self.assertEqual(len(val2), 2)
        
        self.assertEqual(topic2, "test2")
        
        a,b=val2
        self.assertEqual(a, "param1")
        self.assertEqual(b, "param2")
        
    def test_a3_simple_urgent(self):
        """
        Simple urgent, 1 topics, 3msgs
        """
        
        pub("test3", "value3")
        
        self.assertEqual(len(EVENTS), 3)
        
        topic1, val1=EVENTS.pop(0)
        self.assertEqual(topic1, "test3")
        self.assertEqual(val1,   "value3")
        
        topic2, val2=EVENTS.pop(0)
        self.assertEqual(topic2, "test4")
        self.assertEqual(val2,   "value3")
                
        topic3, val3=EVENTS.pop(0)
        self.assertEqual(topic3, "test5")
        self.assertEqual(val3,   "value3")
        
    def test_a4_donotcalldirectly(self):
        """
        Do not call sub function directly
        """
        with self.assertRaises(Exception) as _context:
            on_test1("value1")
            
    def test_z1_2modules(self):
        """
        Simple 2 modules, 1 msg
        """
        import test2
        
        pub("test99", "value1")
        self.assertEqual(len(EVENTS), 2)
        
        topic1, val1=EVENTS.pop(0)
        self.assertEqual(topic1, "test99")
        self.assertEqual(val1,   "value1")
        
        topic2, val2=EVENTS.pop(0)
        self.assertEqual(topic2, "test2/test99")
        self.assertEqual(val2,   "value1")

            
    def test_z2_get_subs(self):
        """
        Test for get_subs
        """
        print
        subs=get_subs()

        for topic in subs:
            liste=subs[topic]
            for sub in liste:
                print "sub: mod=", sub.__module__,", fnc=",sub.__name__

    ## need to put this test towards the end (hence the 'x')
    ## If not, the rest of the tests fail...
    def test_z3_raise(self):
        """
        Raise an exception
        """
        with self.assertRaises(Exception) as _context:
            pub("test6")
        
     
if __name__ == '__main__':
    unittest.main()




"""
    Created on 2013-03-03
    @author: jldupont
"""

import os, sys
ap=os.path.abspath(__file__)
dn=os.path.dirname
base=dn(dn(dn(ap)))
sys.path.insert(0, base)
from subpub import sub

import test1

@sub
def on_test99(param):   
    global EVENTS
    
    test1.push_event(("test2/test99", param))



"""
    Created on 2013-03-03
    @author: jldupont
"""
from functools import wraps

__all__=["sub", "pub", "upub", "get_subs"]

## The interpreter-wide context dictionary
SUBS={}

## The interpreter-wide queue
QUEUE=[]

## For handling re-entrancy during the publishing phase
INPROC=False


def sub(handler_topic):
    """
    Decorator for "topic handler" functions
    
    Keeps track of "topic subscribers"
    
    The name of the input function is used to refer to the topic name e.g.
    
        'on_begin'  ==> topic name='begin'
        
    """
    global SUBS
    
    try:    
        topic=handler_topic.__name__.split("_")[1]
    except:
        raise Exception("Invalid topic handler function name: %s" % handler_topic)
    
    @wraps(handler_topic)
    def message_receiver(*pa):
        return handler_topic(*pa)

    lsubs=SUBS.get(topic, [])
    lsubs.append(message_receiver)
    SUBS[topic]=lsubs
    
    def do_not_call_directly(*p, **k):
        raise Exception("Do not call a 'topic subscriber' function directly: %s" % handler_topic.__name__)
    
    return do_not_call_directly


def pub(topic, *pa):
    """
    Publish function
    
    Queues the message
    """
    global QUEUE
    
    QUEUE.append((topic, pa))
    
    _dopub()
    
    
    
def upub(topic, *pa):
    """
    Urgent publish function
    
    Inserts the message up front i.e. not at the end of the queue
    """
    global QUEUE
    
    QUEUE.insert(0, (topic, pa))
    
    _dopub()

##
## DEBUG  ==============================================================================
##

def get_subs():
    """
    Returns the {topic:[subscribers]
    
    Useful for debugging
    
    Useful for documenting call flows  
    """
    return SUBS


##
## PRIVATE =============================================================================
##    
    
def _dopub():
    """
    Actually performs the publishing
    """
    global INPROC, QUEUE, SUBS
    
    if INPROC:
        return

    INPROC=True
    
    while True:
        try:    msg=QUEUE.pop(0)
        except: break
        
        topic, params=msg
        
        lsubs=SUBS.get(topic, [])
        for sub in lsubs:
            sub(*params)
        
    INPROC=False



#!/usr/bin/env python
"""
    Python "publish-subscribe" framework
    
    Created on 2012-01-19
    @author: Jean-Lou Dupont
"""
__author__  ="Jean-Lou Dupont"
__version__ ="0.1.0"


from distutils.core import setup
from setuptools import find_packages

DESC="""
Overview
--------

This package offers a "publish-subscribe" framework.

The framework can be used to implement basic "actors" where each "actor" is contained in a python module.
The function "upub" can be used to queue a message in front instead of the normal tail. 

Small Example
-------------

   ## Actor 1 in module1.py
   ##
   from subpub import sub, pub
    
   @sub
   def on_topic1(param1):
     print "module1/topic1: ", param1

   @sub
   def on_topic2(param1):
     print "module1/topic2: ", param1
        

   ## Actor 2 in module2.py
   ##
   from subpub import sub, pub
    
   @sub
   def on_topic1(param1):
     print "module2/topic1: ", param1
        
   pub("topic1", "value1")
   pub("topic2", "value2")
    

The example above would yield:

    "module1/topic1: value1"
    "module2/topic1: value1"
    "module1/topic2: value2"
    
"""

setup(name=         'pysubpub',
      version=      __version__,
      description=  'Publish-Subscribe framework',
      author=       __author__,
      author_email= 'jl@jldupont.com',
      url=          'https://github.com/jldupont/pysubpub',
      package_dir=  {'': "src",},
      packages=     find_packages("src"),
      zip_safe=False
      ,install_requires=[
                         ]
      ,long_description=DESC
      ,classifiers=[
                    'Development Status :: 4 - Beta',
                    'Programming Language :: Python',
                    'Intended Audience :: Developers',
                    'Topic :: Utilities',
                    ]
      )

#############################################

f=open("latest", "w")
f.write(str(__version__)+"\n")
f.close()
